"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from flexprice_py import errors, models, utils
from flexprice_py._hooks import HookContext
from flexprice_py.types import OptionalNullable, UNSET
from flexprice_py.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional, Union


class Coupons(BaseSDK):
    def create_coupon(
        self,
        *,
        cadence: models.components.CouponCadence,
        name: str,
        type_: models.components.CouponType,
        amount_off: Optional[str] = None,
        currency: Optional[str] = None,
        duration_in_periods: Optional[int] = None,
        max_redemptions: Optional[int] = None,
        metadata: Optional[Dict[str, str]] = None,
        percentage_off: Optional[str] = None,
        redeem_after: Optional[str] = None,
        redeem_before: Optional[str] = None,
        rules: Optional[Dict[str, Any]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoCouponResponse:
        r"""Create coupon

        Use when creating a discount (e.g. promo code or referral). Ideal for percent or fixed value, with optional validity and usage limits.

        :param cadence:
        :param name:
        :param type:
        :param amount_off:
        :param currency:
        :param duration_in_periods:
        :param max_redemptions:
        :param metadata:
        :param percentage_off:
        :param redeem_after:
        :param redeem_before:
        :param rules:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoCreateCouponRequest(
            amount_off=amount_off,
            cadence=cadence,
            currency=currency,
            duration_in_periods=duration_in_periods,
            max_redemptions=max_redemptions,
            metadata=metadata,
            name=name,
            percentage_off=percentage_off,
            redeem_after=redeem_after,
            redeem_before=redeem_before,
            rules=rules,
            type=type_,
        )

        req = self._build_request(
            method="POST",
            path="/coupons",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.DtoCreateCouponRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createCoupon",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.components.DtoCouponResponse, http_res
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def create_coupon_async(
        self,
        *,
        cadence: models.components.CouponCadence,
        name: str,
        type_: models.components.CouponType,
        amount_off: Optional[str] = None,
        currency: Optional[str] = None,
        duration_in_periods: Optional[int] = None,
        max_redemptions: Optional[int] = None,
        metadata: Optional[Dict[str, str]] = None,
        percentage_off: Optional[str] = None,
        redeem_after: Optional[str] = None,
        redeem_before: Optional[str] = None,
        rules: Optional[Dict[str, Any]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoCouponResponse:
        r"""Create coupon

        Use when creating a discount (e.g. promo code or referral). Ideal for percent or fixed value, with optional validity and usage limits.

        :param cadence:
        :param name:
        :param type:
        :param amount_off:
        :param currency:
        :param duration_in_periods:
        :param max_redemptions:
        :param metadata:
        :param percentage_off:
        :param redeem_after:
        :param redeem_before:
        :param rules:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoCreateCouponRequest(
            amount_off=amount_off,
            cadence=cadence,
            currency=currency,
            duration_in_periods=duration_in_periods,
            max_redemptions=max_redemptions,
            metadata=metadata,
            name=name,
            percentage_off=percentage_off,
            redeem_after=redeem_after,
            redeem_before=redeem_before,
            rules=rules,
            type=type_,
        )

        req = self._build_request_async(
            method="POST",
            path="/coupons",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.DtoCreateCouponRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createCoupon",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.components.DtoCouponResponse, http_res
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def query_coupon(
        self,
        *,
        coupon_ids: Optional[List[str]] = None,
        expand: Optional[str] = None,
        filters: Optional[
            Union[
                List[models.components.FilterCondition],
                List[models.components.FilterConditionTypedDict],
            ]
        ] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.components.CouponFilterOrder] = None,
        sort: Optional[
            Union[
                List[models.components.SortCondition],
                List[models.components.SortConditionTypedDict],
            ]
        ] = None,
        status: Optional[models.components.Status] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListCouponsResponse:
        r"""Query coupons

        Use when listing or searching coupons (e.g. promo management). Returns a paginated list; supports filtering and sorting.

        :param coupon_ids:
        :param expand:
        :param filters:
        :param limit:
        :param offset:
        :param order:
        :param sort:
        :param status:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.CouponFilter(
            coupon_ids=coupon_ids,
            expand=expand,
            filters=utils.get_pydantic_model(
                filters, Optional[List[models.components.FilterCondition]]
            ),
            limit=limit,
            offset=offset,
            order=order,
            sort=utils.get_pydantic_model(
                sort, Optional[List[models.components.SortCondition]]
            ),
            status=status,
        )

        req = self._build_request(
            method="POST",
            path="/coupons/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.CouponFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryCoupon",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListCouponsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def query_coupon_async(
        self,
        *,
        coupon_ids: Optional[List[str]] = None,
        expand: Optional[str] = None,
        filters: Optional[
            Union[
                List[models.components.FilterCondition],
                List[models.components.FilterConditionTypedDict],
            ]
        ] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.components.CouponFilterOrder] = None,
        sort: Optional[
            Union[
                List[models.components.SortCondition],
                List[models.components.SortConditionTypedDict],
            ]
        ] = None,
        status: Optional[models.components.Status] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListCouponsResponse:
        r"""Query coupons

        Use when listing or searching coupons (e.g. promo management). Returns a paginated list; supports filtering and sorting.

        :param coupon_ids:
        :param expand:
        :param filters:
        :param limit:
        :param offset:
        :param order:
        :param sort:
        :param status:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.CouponFilter(
            coupon_ids=coupon_ids,
            expand=expand,
            filters=utils.get_pydantic_model(
                filters, Optional[List[models.components.FilterCondition]]
            ),
            limit=limit,
            offset=offset,
            order=order,
            sort=utils.get_pydantic_model(
                sort, Optional[List[models.components.SortCondition]]
            ),
            status=status,
        )

        req = self._build_request_async(
            method="POST",
            path="/coupons/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.CouponFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryCoupon",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListCouponsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def get_coupon(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoCouponResponse:
        r"""Get coupon

        Use when you need to load a single coupon (e.g. for display or to validate a code).

        :param id: Coupon ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetCouponRequest(
            id=id,
        )

        req = self._build_request(
            method="GET",
            path="/coupons/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getCoupon",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoCouponResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_coupon_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoCouponResponse:
        r"""Get coupon

        Use when you need to load a single coupon (e.g. for display or to validate a code).

        :param id: Coupon ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetCouponRequest(
            id=id,
        )

        req = self._build_request_async(
            method="GET",
            path="/coupons/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getCoupon",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoCouponResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def update_coupon(
        self,
        *,
        id: str,
        metadata: Optional[Dict[str, str]] = None,
        name: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoCouponResponse:
        r"""Update coupon

        Use when changing coupon config (e.g. value, validity, or usage limits).

        :param id: Coupon ID
        :param metadata:
        :param name:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.UpdateCouponRequest(
            id=id,
            body=models.components.DtoUpdateCouponRequest(
                metadata=metadata,
                name=name,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/coupons/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.components.DtoUpdateCouponRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateCoupon",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoCouponResponse, http_res
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def update_coupon_async(
        self,
        *,
        id: str,
        metadata: Optional[Dict[str, str]] = None,
        name: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoCouponResponse:
        r"""Update coupon

        Use when changing coupon config (e.g. value, validity, or usage limits).

        :param id: Coupon ID
        :param metadata:
        :param name:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.UpdateCouponRequest(
            id=id,
            body=models.components.DtoUpdateCouponRequest(
                metadata=metadata,
                name=name,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/coupons/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.components.DtoUpdateCouponRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateCoupon",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoCouponResponse, http_res
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def delete_coupon(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, str]:
        r"""Delete coupon

        Use when retiring a coupon (e.g. campaign ended). Returns 200 with success message.

        :param id: Coupon ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.DeleteCouponRequest(
            id=id,
        )

        req = self._build_request(
            method="DELETE",
            path="/coupons/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deleteCoupon",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, str], http_res)
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def delete_coupon_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, str]:
        r"""Delete coupon

        Use when retiring a coupon (e.g. campaign ended). Returns 200 with success message.

        :param id: Coupon ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.DeleteCouponRequest(
            id=id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/coupons/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deleteCoupon",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, str], http_res)
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class PricePriceTierTypedDict(TypedDict):
    flat_amount: NotRequired[str]
    r"""flat_amount is the flat amount for the given tier (optional)
    Applied on top of unit_amount*quantity. Useful for cases like \"2.7$ + 5c\" 
    """
    unit_amount: NotRequired[str]
    r"""unit_amount is the amount per unit for the given tier"""
    up_to: NotRequired[int]
    r"""up_to is the quantity up to which this tier applies. It is null for the last tier.
    IMPORTANT: Tier boundaries are INCLUSIVE.
    - If up_to is 1000, then quantity less than or equal to 1000 belongs to this tier
    - This behavior is consistent across both VOLUME and SLAB tier modes
    """


class PricePriceTier(BaseModel):
    flat_amount: Optional[str] = None
    r"""flat_amount is the flat amount for the given tier (optional)
    Applied on top of unit_amount*quantity. Useful for cases like \"2.7$ + 5c\" 
    """

    unit_amount: Optional[str] = None
    r"""unit_amount is the amount per unit for the given tier"""

    up_to: Optional[int] = None
    r"""up_to is the quantity up to which this tier applies. It is null for the last tier.
    IMPORTANT: Tier boundaries are INCLUSIVE.
    - If up_to is 1000, then quantity less than or equal to 1000 belongs to this tier
    - This behavior is consistent across both VOLUME and SLAB tier modes
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["flat_amount", "unit_amount", "up_to"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_aggregatedentitlement import (
    DtoAggregatedEntitlement,
    DtoAggregatedEntitlementTypedDict,
)
from .dto_entitlementsource import DtoEntitlementSource, DtoEntitlementSourceTypedDict
from .dto_featureresponse import DtoFeatureResponse, DtoFeatureResponseTypedDict
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoAggregatedFeatureTypedDict(TypedDict):
    entitlement: NotRequired[DtoAggregatedEntitlementTypedDict]
    feature: NotRequired[DtoFeatureResponseTypedDict]
    sources: NotRequired[List[DtoEntitlementSourceTypedDict]]


class DtoAggregatedFeature(BaseModel):
    entitlement: Optional[DtoAggregatedEntitlement] = None

    feature: Optional[DtoFeatureResponse] = None

    sources: Optional[List[DtoEntitlementSource]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["entitlement", "feature", "sources"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .alertentitytype import AlertEntityType
from .alertinfo import AlertInfo, AlertInfoTypedDict
from .alertstate import AlertState
from .alerttype import AlertType
from .dto_customerresponse import DtoCustomerResponse, DtoCustomerResponseTypedDict
from .dto_featureresponse import DtoFeatureResponse, DtoFeatureResponseTypedDict
from .dto_walletresponse import DtoWalletResponse, DtoWalletResponseTypedDict
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoAlertLogResponseTypedDict(TypedDict):
    alert_info: NotRequired[AlertInfoTypedDict]
    alert_status: NotRequired[AlertState]
    alert_type: NotRequired[AlertType]
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    customer: NotRequired[DtoCustomerResponseTypedDict]
    r"""Customer response object containing all customer information"""
    customer_id: NotRequired[str]
    entity_id: NotRequired[str]
    entity_type: NotRequired[AlertEntityType]
    environment_id: NotRequired[str]
    feature: NotRequired[DtoFeatureResponseTypedDict]
    id: NotRequired[str]
    parent_entity_id: NotRequired[str]
    parent_entity_type: NotRequired[str]
    status: NotRequired[str]
    tenant_id: NotRequired[str]
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]
    wallet: NotRequired[DtoWalletResponseTypedDict]


class DtoAlertLogResponse(BaseModel):
    alert_info: Optional[AlertInfo] = None

    alert_status: Optional[AlertState] = None

    alert_type: Optional[AlertType] = None

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    customer: Optional[DtoCustomerResponse] = None
    r"""Customer response object containing all customer information"""

    customer_id: Optional[str] = None

    entity_id: Optional[str] = None

    entity_type: Optional[AlertEntityType] = None

    environment_id: Optional[str] = None

    feature: Optional[DtoFeatureResponse] = None

    id: Optional[str] = None

    parent_entity_id: Optional[str] = None

    parent_entity_type: Optional[str] = None

    status: Optional[str] = None

    tenant_id: Optional[str] = None

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    wallet: Optional[DtoWalletResponse] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "alert_info",
                "alert_status",
                "alert_type",
                "created_at",
                "created_by",
                "customer",
                "customer_id",
                "entity_id",
                "entity_type",
                "environment_id",
                "feature",
                "id",
                "parent_entity_id",
                "parent_entity_type",
                "status",
                "tenant_id",
                "updated_at",
                "updated_by",
                "wallet",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoGetPreviewInvoiceRequestTypedDict(TypedDict):
    subscription_id: str
    r"""subscription_id is the unique identifier of the subscription to preview invoice for"""
    period_end: NotRequired[str]
    r"""period_end is the optional end date of the period to preview"""
    period_start: NotRequired[str]
    r"""period_start is the optional start date of the period to preview"""


class DtoGetPreviewInvoiceRequest(BaseModel):
    subscription_id: str
    r"""subscription_id is the unique identifier of the subscription to preview invoice for"""

    period_end: Optional[str] = None
    r"""period_end is the optional end date of the period to preview"""

    period_start: Optional[str] = None
    r"""period_start is the optional start date of the period to preview"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["period_end", "period_start"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .creditnote_creditnotelineitem import (
    CreditnoteCreditNoteLineItem,
    CreditnoteCreditNoteLineItemTypedDict,
)
from .creditnotereason import CreditNoteReason
from .creditnotestatus import CreditNoteStatus
from .creditnotetype import CreditNoteType
from .dto_invoiceresponse import DtoInvoiceResponse, DtoInvoiceResponseTypedDict
from .dto_subscriptionresponse import (
    DtoSubscriptionResponse,
    DtoSubscriptionResponseTypedDict,
)
from .github_com_flexprice_flexprice_internal_domain_customer_customer import (
    GithubComFlexpriceFlexpriceInternalDomainCustomerCustomer,
    GithubComFlexpriceFlexpriceInternalDomainCustomerCustomerTypedDict,
)
from .paymentstatus import PaymentStatus
from .status import Status
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCreditNoteResponseTypedDict(TypedDict):
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    credit_note_number: NotRequired[str]
    r"""credit_note_number is the unique identifier for credit notes"""
    credit_note_status: NotRequired[CreditNoteStatus]
    credit_note_type: NotRequired[CreditNoteType]
    currency: NotRequired[str]
    r"""currency is the three-letter ISO currency code (e.g., USD, EUR) for the credit note"""
    customer: NotRequired[
        GithubComFlexpriceFlexpriceInternalDomainCustomerCustomerTypedDict
    ]
    customer_id: NotRequired[str]
    r"""customer_id is the unique identifier of the customer who owns this credit note"""
    environment_id: NotRequired[str]
    r"""environment_id is the unique identifier of the environment this credit note belongs to"""
    finalized_at: NotRequired[str]
    r"""finalized_at is the timestamp when the credit note was finalized"""
    id: NotRequired[str]
    r"""id is the unique identifier for the credit note"""
    idempotency_key: NotRequired[str]
    r"""idempotency_key is an optional key used to prevent duplicate credit note creation"""
    invoice: NotRequired[DtoInvoiceResponseTypedDict]
    invoice_id: NotRequired[str]
    r"""invoice_id is the id of the invoice resource that this credit note is applied to"""
    line_items: NotRequired[List[CreditnoteCreditNoteLineItemTypedDict]]
    r"""line_items contains all of the line items associated with this credit note"""
    memo: NotRequired[str]
    r"""memo is an optional memo supplied on the credit note"""
    metadata: NotRequired[Dict[str, str]]
    reason: NotRequired[CreditNoteReason]
    refund_status: NotRequired[PaymentStatus]
    status: NotRequired[Status]
    subscription: NotRequired[DtoSubscriptionResponseTypedDict]
    subscription_id: NotRequired[str]
    r"""subscription_id is the optional unique identifier of the subscription related to this credit note"""
    tenant_id: NotRequired[str]
    total_amount: NotRequired[str]
    r"""total_amount is the total including creditable invoice-level discounts or minimums, and tax"""
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]
    voided_at: NotRequired[str]
    r"""voided_at is the timestamp when the credit note was voided"""


class DtoCreditNoteResponse(BaseModel):
    created_at: Optional[str] = None

    created_by: Optional[str] = None

    credit_note_number: Optional[str] = None
    r"""credit_note_number is the unique identifier for credit notes"""

    credit_note_status: Optional[CreditNoteStatus] = None

    credit_note_type: Optional[CreditNoteType] = None

    currency: Optional[str] = None
    r"""currency is the three-letter ISO currency code (e.g., USD, EUR) for the credit note"""

    customer: Optional[GithubComFlexpriceFlexpriceInternalDomainCustomerCustomer] = None

    customer_id: Optional[str] = None
    r"""customer_id is the unique identifier of the customer who owns this credit note"""

    environment_id: Optional[str] = None
    r"""environment_id is the unique identifier of the environment this credit note belongs to"""

    finalized_at: Optional[str] = None
    r"""finalized_at is the timestamp when the credit note was finalized"""

    id: Optional[str] = None
    r"""id is the unique identifier for the credit note"""

    idempotency_key: Optional[str] = None
    r"""idempotency_key is an optional key used to prevent duplicate credit note creation"""

    invoice: Optional[DtoInvoiceResponse] = None

    invoice_id: Optional[str] = None
    r"""invoice_id is the id of the invoice resource that this credit note is applied to"""

    line_items: Optional[List[CreditnoteCreditNoteLineItem]] = None
    r"""line_items contains all of the line items associated with this credit note"""

    memo: Optional[str] = None
    r"""memo is an optional memo supplied on the credit note"""

    metadata: Optional[Dict[str, str]] = None

    reason: Optional[CreditNoteReason] = None

    refund_status: Optional[PaymentStatus] = None

    status: Optional[Status] = None

    subscription: Optional[DtoSubscriptionResponse] = None

    subscription_id: Optional[str] = None
    r"""subscription_id is the optional unique identifier of the subscription related to this credit note"""

    tenant_id: Optional[str] = None

    total_amount: Optional[str] = None
    r"""total_amount is the total including creditable invoice-level discounts or minimums, and tax"""

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    voided_at: Optional[str] = None
    r"""voided_at is the timestamp when the credit note was voided"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "created_at",
                "created_by",
                "credit_note_number",
                "credit_note_status",
                "credit_note_type",
                "currency",
                "customer",
                "customer_id",
                "environment_id",
                "finalized_at",
                "id",
                "idempotency_key",
                "invoice",
                "invoice_id",
                "line_items",
                "memo",
                "metadata",
                "reason",
                "refund_status",
                "status",
                "subscription",
                "subscription_id",
                "tenant_id",
                "total_amount",
                "updated_at",
                "updated_by",
                "voided_at",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import UnrecognizedStr
from typing import Literal, Union


EntityType = Union[
    Literal[
        "EVENTS",
        "PRICES",
        "CUSTOMERS",
        "FEATURES",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .github_com_flexprice_flexprice_internal_domain_coupon_coupon import (
    GithubComFlexpriceFlexpriceInternalDomainCouponCoupon,
    GithubComFlexpriceFlexpriceInternalDomainCouponCouponTypedDict,
)
from .status import Status
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCouponAssociationResponseTypedDict(TypedDict):
    coupon: NotRequired[GithubComFlexpriceFlexpriceInternalDomainCouponCouponTypedDict]
    coupon_id: NotRequired[str]
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    end_date: NotRequired[str]
    r"""Optional"""
    environment_id: NotRequired[str]
    id: NotRequired[str]
    metadata: NotRequired[Dict[str, str]]
    start_date: NotRequired[str]
    status: NotRequired[Status]
    subscription_id: NotRequired[str]
    r"""Mandatory"""
    subscription_line_item_id: NotRequired[str]
    r"""Optional"""
    subscription_phase_id: NotRequired[str]
    r"""Optional"""
    tenant_id: NotRequired[str]
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]


class DtoCouponAssociationResponse(BaseModel):
    coupon: Optional[GithubComFlexpriceFlexpriceInternalDomainCouponCoupon] = None

    coupon_id: Optional[str] = None

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    end_date: Optional[str] = None
    r"""Optional"""

    environment_id: Optional[str] = None

    id: Optional[str] = None

    metadata: Optional[Dict[str, str]] = None

    start_date: Optional[str] = None

    status: Optional[Status] = None

    subscription_id: Optional[str] = None
    r"""Mandatory"""

    subscription_line_item_id: Optional[str] = None
    r"""Optional"""

    subscription_phase_id: Optional[str] = None
    r"""Optional"""

    tenant_id: Optional[str] = None

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "coupon",
                "coupon_id",
                "created_at",
                "created_by",
                "end_date",
                "environment_id",
                "id",
                "metadata",
                "start_date",
                "status",
                "subscription_id",
                "subscription_line_item_id",
                "subscription_phase_id",
                "tenant_id",
                "updated_at",
                "updated_by",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoProrationDetailTypedDict(TypedDict):
    charge_amount: NotRequired[str]
    credit_amount: NotRequired[str]
    description: NotRequired[str]
    line_item_id: NotRequired[str]
    original_amount: NotRequired[str]
    plan_name: NotRequired[str]
    price_id: NotRequired[str]
    proration_days: NotRequired[int]


class DtoProrationDetail(BaseModel):
    charge_amount: Optional[str] = None

    credit_amount: Optional[str] = None

    description: Optional[str] = None

    line_item_id: Optional[str] = None

    original_amount: Optional[str] = None

    plan_name: Optional[str] = None

    price_id: Optional[str] = None

    proration_days: Optional[int] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "charge_amount",
                "credit_amount",
                "description",
                "line_item_id",
                "original_amount",
                "plan_name",
                "price_id",
                "proration_days",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_lineitemcommitmentconfig import (
    DtoLineItemCommitmentConfig,
    DtoLineItemCommitmentConfigTypedDict,
)
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoAddAddonRequestTypedDict(TypedDict):
    addon_id: str
    subscription_id: str
    line_item_commitments: NotRequired[Dict[str, DtoLineItemCommitmentConfigTypedDict]]
    r"""LineItemCommitments allows setting commitment configuration per addon line item (keyed by price_id)"""
    metadata: NotRequired[Dict[str, Any]]
    start_date: NotRequired[str]


class DtoAddAddonRequest(BaseModel):
    addon_id: str

    subscription_id: str

    line_item_commitments: Optional[Dict[str, DtoLineItemCommitmentConfig]] = None
    r"""LineItemCommitments allows setting commitment configuration per addon line item (keyed by price_id)"""

    metadata: Optional[Dict[str, Any]] = None

    start_date: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["line_item_commitments", "metadata", "start_date"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .aggregationtype import AggregationType
from .commitmentinfo import CommitmentInfo, CommitmentInfoTypedDict
from .dto_priceresponse import DtoPriceResponse, DtoPriceResponseTypedDict
from .dto_usageanalyticpoint import (
    DtoUsageAnalyticPoint,
    DtoUsageAnalyticPointTypedDict,
)
from .github_com_flexprice_flexprice_internal_domain_addon_addon import (
    GithubComFlexpriceFlexpriceInternalDomainAddonAddon,
    GithubComFlexpriceFlexpriceInternalDomainAddonAddonTypedDict,
)
from .github_com_flexprice_flexprice_internal_domain_feature_feature import (
    GithubComFlexpriceFlexpriceInternalDomainFeatureFeature,
    GithubComFlexpriceFlexpriceInternalDomainFeatureFeatureTypedDict,
)
from .github_com_flexprice_flexprice_internal_domain_plan_plan import (
    GithubComFlexpriceFlexpriceInternalDomainPlanPlan,
    GithubComFlexpriceFlexpriceInternalDomainPlanPlanTypedDict,
)
from .meter_meter import MeterMeter, MeterMeterTypedDict
from .subscription_subscriptionlineitem import (
    SubscriptionSubscriptionLineItem,
    SubscriptionSubscriptionLineItemTypedDict,
)
from .windowsize import WindowSize
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoUsageAnalyticItemTypedDict(TypedDict):
    add_on_id: NotRequired[str]
    addon: NotRequired[GithubComFlexpriceFlexpriceInternalDomainAddonAddonTypedDict]
    aggregation_type: NotRequired[AggregationType]
    commitment_info: NotRequired[CommitmentInfoTypedDict]
    currency: NotRequired[str]
    event_count: NotRequired[int]
    r"""Number of events that contributed to this aggregation"""
    event_name: NotRequired[str]
    feature: NotRequired[
        GithubComFlexpriceFlexpriceInternalDomainFeatureFeatureTypedDict
    ]
    feature_id: NotRequired[str]
    meter: NotRequired[MeterMeterTypedDict]
    meter_id: NotRequired[str]
    r"""Meter ID"""
    name: NotRequired[str]
    plan: NotRequired[GithubComFlexpriceFlexpriceInternalDomainPlanPlanTypedDict]
    plan_id: NotRequired[str]
    points: NotRequired[List[DtoUsageAnalyticPointTypedDict]]
    price: NotRequired[DtoPriceResponseTypedDict]
    price_id: NotRequired[str]
    r"""Price ID used for this usage"""
    properties: NotRequired[Dict[str, str]]
    r"""Stores property values for flexible grouping (e.g., org_id -> \"org123\")"""
    source: NotRequired[str]
    sources: NotRequired[List[str]]
    r"""List of sources when not grouping by source"""
    sub_line_item_id: NotRequired[str]
    r"""Subscription line item ID"""
    subscription_id: NotRequired[str]
    r"""Subscription ID"""
    subscription_line_item: NotRequired[SubscriptionSubscriptionLineItemTypedDict]
    total_cost: NotRequired[str]
    total_usage: NotRequired[str]
    unit: NotRequired[str]
    unit_plural: NotRequired[str]
    window_size: NotRequired[WindowSize]


class DtoUsageAnalyticItem(BaseModel):
    add_on_id: Optional[str] = None

    addon: Optional[GithubComFlexpriceFlexpriceInternalDomainAddonAddon] = None

    aggregation_type: Optional[AggregationType] = None

    commitment_info: Optional[CommitmentInfo] = None

    currency: Optional[str] = None

    event_count: Optional[int] = None
    r"""Number of events that contributed to this aggregation"""

    event_name: Optional[str] = None

    feature: Optional[GithubComFlexpriceFlexpriceInternalDomainFeatureFeature] = None

    feature_id: Optional[str] = None

    meter: Optional[MeterMeter] = None

    meter_id: Optional[str] = None
    r"""Meter ID"""

    name: Optional[str] = None

    plan: Optional[GithubComFlexpriceFlexpriceInternalDomainPlanPlan] = None

    plan_id: Optional[str] = None

    points: Optional[List[DtoUsageAnalyticPoint]] = None

    price: Optional[DtoPriceResponse] = None

    price_id: Optional[str] = None
    r"""Price ID used for this usage"""

    properties: Optional[Dict[str, str]] = None
    r"""Stores property values for flexible grouping (e.g., org_id -> \"org123\")"""

    source: Optional[str] = None

    sources: Optional[List[str]] = None
    r"""List of sources when not grouping by source"""

    sub_line_item_id: Optional[str] = None
    r"""Subscription line item ID"""

    subscription_id: Optional[str] = None
    r"""Subscription ID"""

    subscription_line_item: Optional[SubscriptionSubscriptionLineItem] = None

    total_cost: Optional[str] = None

    total_usage: Optional[str] = None

    unit: Optional[str] = None

    unit_plural: Optional[str] = None

    window_size: Optional[WindowSize] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "add_on_id",
                "addon",
                "aggregation_type",
                "commitment_info",
                "currency",
                "event_count",
                "event_name",
                "feature",
                "feature_id",
                "meter",
                "meter_id",
                "name",
                "plan",
                "plan_id",
                "points",
                "price",
                "price_id",
                "properties",
                "source",
                "sources",
                "sub_line_item_id",
                "subscription_id",
                "subscription_line_item",
                "total_cost",
                "total_usage",
                "unit",
                "unit_plural",
                "window_size",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from flexprice_py.utils import FieldMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


GetTaxRatesOrder = Literal[
    "asc",
    "desc",
]


Scope = Literal[
    "INTERNAL",
    "EXTERNAL",
    "ONETIME",
]


GetTaxRatesStatus = Literal[
    "published",
    "deleted",
    "archived",
]


class GetTaxRatesRequestTypedDict(TypedDict):
    end_time: NotRequired[str]
    expand: NotRequired[str]
    limit: NotRequired[int]
    offset: NotRequired[int]
    order: NotRequired[GetTaxRatesOrder]
    scope: NotRequired[Scope]
    start_time: NotRequired[str]
    status: NotRequired[GetTaxRatesStatus]
    taxrate_codes: NotRequired[List[str]]
    taxrate_ids: NotRequired[List[str]]


class GetTaxRatesRequest(BaseModel):
    end_time: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    expand: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    offset: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    order: Annotated[
        Optional[GetTaxRatesOrder],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    scope: Annotated[
        Optional[Scope],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    start_time: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    status: Annotated[
        Optional[GetTaxRatesStatus],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    taxrate_codes: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    taxrate_ids: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "end_time",
                "expand",
                "limit",
                "offset",
                "order",
                "scope",
                "start_time",
                "status",
                "taxrate_codes",
                "taxrate_ids",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .billingperiod import BillingPeriod
from .commitmenttype import CommitmentType
from .invoicecadence import InvoiceCadence
from .price_price import PricePrice, PricePriceTypedDict
from .pricetype import PriceType
from .status import Status
from .subscriptionlineitementitytype import SubscriptionLineItemEntityType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class SubscriptionSubscriptionLineItemTypedDict(TypedDict):
    billing_period: NotRequired[BillingPeriod]
    billing_period_count: NotRequired[int]
    r"""from price at create; default 1"""
    commitment_amount: NotRequired[str]
    r"""Commitment fields"""
    commitment_duration: NotRequired[BillingPeriod]
    commitment_overage_factor: NotRequired[str]
    commitment_quantity: NotRequired[str]
    commitment_true_up_enabled: NotRequired[bool]
    commitment_type: NotRequired[CommitmentType]
    commitment_windowed: NotRequired[bool]
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    currency: NotRequired[str]
    customer_id: NotRequired[str]
    display_name: NotRequired[str]
    end_date: NotRequired[str]
    entity_id: NotRequired[str]
    entity_type: NotRequired[SubscriptionLineItemEntityType]
    environment_id: NotRequired[str]
    id: NotRequired[str]
    invoice_cadence: NotRequired[InvoiceCadence]
    metadata: NotRequired[Dict[str, str]]
    meter_display_name: NotRequired[str]
    meter_id: NotRequired[str]
    plan_display_name: NotRequired[str]
    price: NotRequired[PricePriceTypedDict]
    price_id: NotRequired[str]
    price_type: NotRequired[PriceType]
    price_unit: NotRequired[str]
    price_unit_id: NotRequired[str]
    quantity: NotRequired[str]
    start_date: NotRequired[str]
    status: NotRequired[Status]
    subscription_id: NotRequired[str]
    subscription_phase_id: NotRequired[str]
    tenant_id: NotRequired[str]
    trial_period: NotRequired[int]
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]


class SubscriptionSubscriptionLineItem(BaseModel):
    billing_period: Optional[BillingPeriod] = None

    billing_period_count: Optional[int] = None
    r"""from price at create; default 1"""

    commitment_amount: Optional[str] = None
    r"""Commitment fields"""

    commitment_duration: Optional[BillingPeriod] = None

    commitment_overage_factor: Optional[str] = None

    commitment_quantity: Optional[str] = None

    commitment_true_up_enabled: Optional[bool] = None

    commitment_type: Optional[CommitmentType] = None

    commitment_windowed: Optional[bool] = None

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    currency: Optional[str] = None

    customer_id: Optional[str] = None

    display_name: Optional[str] = None

    end_date: Optional[str] = None

    entity_id: Optional[str] = None

    entity_type: Optional[SubscriptionLineItemEntityType] = None

    environment_id: Optional[str] = None

    id: Optional[str] = None

    invoice_cadence: Optional[InvoiceCadence] = None

    metadata: Optional[Dict[str, str]] = None

    meter_display_name: Optional[str] = None

    meter_id: Optional[str] = None

    plan_display_name: Optional[str] = None

    price: Optional[PricePrice] = None

    price_id: Optional[str] = None

    price_type: Optional[PriceType] = None

    price_unit: Optional[str] = None

    price_unit_id: Optional[str] = None

    quantity: Optional[str] = None

    start_date: Optional[str] = None

    status: Optional[Status] = None

    subscription_id: Optional[str] = None

    subscription_phase_id: Optional[str] = None

    tenant_id: Optional[str] = None

    trial_period: Optional[int] = None

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "billing_period",
                "billing_period_count",
                "commitment_amount",
                "commitment_duration",
                "commitment_overage_factor",
                "commitment_quantity",
                "commitment_true_up_enabled",
                "commitment_type",
                "commitment_windowed",
                "created_at",
                "created_by",
                "currency",
                "customer_id",
                "display_name",
                "end_date",
                "entity_id",
                "entity_type",
                "environment_id",
                "id",
                "invoice_cadence",
                "metadata",
                "meter_display_name",
                "meter_id",
                "plan_display_name",
                "price",
                "price_id",
                "price_type",
                "price_unit",
                "price_unit_id",
                "quantity",
                "start_date",
                "status",
                "subscription_id",
                "subscription_phase_id",
                "tenant_id",
                "trial_period",
                "updated_at",
                "updated_by",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel
from typing_extensions import TypedDict


class DtoUpdateScheduledTaskRequestTypedDict(TypedDict):
    enabled: bool


class DtoUpdateScheduledTaskRequest(BaseModel):
    enabled: bool



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .couponcadence import CouponCadence
from .coupontype import CouponType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCreateCouponRequestTypedDict(TypedDict):
    cadence: CouponCadence
    name: str
    type: CouponType
    amount_off: NotRequired[str]
    currency: NotRequired[str]
    duration_in_periods: NotRequired[int]
    max_redemptions: NotRequired[int]
    metadata: NotRequired[Dict[str, str]]
    percentage_off: NotRequired[str]
    redeem_after: NotRequired[str]
    redeem_before: NotRequired[str]
    rules: NotRequired[Dict[str, Any]]


class DtoCreateCouponRequest(BaseModel):
    cadence: CouponCadence

    name: str

    type: CouponType

    amount_off: Optional[str] = None

    currency: Optional[str] = None

    duration_in_periods: Optional[int] = None

    max_redemptions: Optional[int] = None

    metadata: Optional[Dict[str, str]] = None

    percentage_off: Optional[str] = None

    redeem_after: Optional[str] = None

    redeem_before: Optional[str] = None

    rules: Optional[Dict[str, Any]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "amount_off",
                "currency",
                "duration_in_periods",
                "max_redemptions",
                "metadata",
                "percentage_off",
                "redeem_after",
                "redeem_before",
                "rules",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import UnrecognizedStr
from typing import Literal, Union


BillingPeriod = Union[
    Literal[
        "MONTHLY",
        "ANNUAL",
        "WEEKLY",
        "DAILY",
        "QUARTERLY",
        "HALF_YEARLY",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_subscriptionpauseresponse import (
    DtoSubscriptionPauseResponse,
    DtoSubscriptionPauseResponseTypedDict,
)
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoListSubscriptionPausesResponseTypedDict(TypedDict):
    r"""Response object for listing subscription pauses with total count"""

    items: NotRequired[List[DtoSubscriptionPauseResponseTypedDict]]
    r"""List of subscription pause objects
    @Description Array of subscription pauses
    """
    total: NotRequired[int]
    r"""Total number of pauses
    @Description Total count of subscription pauses in the response
    """


class DtoListSubscriptionPausesResponse(BaseModel):
    r"""Response object for listing subscription pauses with total count"""

    items: Optional[List[DtoSubscriptionPauseResponse]] = None
    r"""List of subscription pause objects
    @Description Array of subscription pauses
    """

    total: Optional[int] = None
    r"""Total number of pauses
    @Description Total count of subscription pauses in the response
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["items", "total"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .alertsettings import AlertSettings, AlertSettingsTypedDict
from .alertstate import AlertState
from .autotopup import AutoTopup, AutoTopupTypedDict
from .creditbreakdown import CreditBreakdown, CreditBreakdownTypedDict
from .status import Status
from .walletconfig import WalletConfig, WalletConfigTypedDict
from .walletstatus import WalletStatus
from .wallettype import WalletType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoWalletBalanceResponseTypedDict(TypedDict):
    alert_settings: NotRequired[AlertSettingsTypedDict]
    alert_state: NotRequired[AlertState]
    auto_topup: NotRequired[AutoTopupTypedDict]
    balance: NotRequired[str]
    balance_updated_at: NotRequired[str]
    config: NotRequired[WalletConfigTypedDict]
    conversion_rate: NotRequired[str]
    r"""amount in the currency =  number of credits * conversion_rate
    ex if conversion_rate is 1, then 1 USD = 1 credit
    ex if conversion_rate is 2, then 1 USD = 0.5 credits
    ex if conversion_rate is 0.5, then 1 USD = 2 credits
    """
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    credit_balance: NotRequired[str]
    credits_available_breakdown: NotRequired[CreditBreakdownTypedDict]
    currency: NotRequired[str]
    current_period_usage: NotRequired[str]
    customer_id: NotRequired[str]
    description: NotRequired[str]
    environment_id: NotRequired[str]
    id: NotRequired[str]
    metadata: NotRequired[Dict[str, str]]
    name: NotRequired[str]
    real_time_balance: NotRequired[str]
    real_time_credit_balance: NotRequired[str]
    status: NotRequired[Status]
    tenant_id: NotRequired[str]
    topup_conversion_rate: NotRequired[str]
    r"""topup_conversion_rate is the conversion rate for the topup to the currency
    ex if topup_conversion_rate is 1, then 1 USD = 1 credit
    ex if topup_conversion_rate is 2, then 1 USD = 0.5 credits
    ex if topup_conversion_rate is 0.5, then 1 USD = 2 credits
    """
    unpaid_invoices_amount: NotRequired[str]
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]
    wallet_status: NotRequired[WalletStatus]
    wallet_type: NotRequired[WalletType]


class DtoWalletBalanceResponse(BaseModel):
    alert_settings: Optional[AlertSettings] = None

    alert_state: Optional[AlertState] = None

    auto_topup: Optional[AutoTopup] = None

    balance: Optional[str] = None

    balance_updated_at: Optional[str] = None

    config: Optional[WalletConfig] = None

    conversion_rate: Optional[str] = None
    r"""amount in the currency =  number of credits * conversion_rate
    ex if conversion_rate is 1, then 1 USD = 1 credit
    ex if conversion_rate is 2, then 1 USD = 0.5 credits
    ex if conversion_rate is 0.5, then 1 USD = 2 credits
    """

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    credit_balance: Optional[str] = None

    credits_available_breakdown: Optional[CreditBreakdown] = None

    currency: Optional[str] = None

    current_period_usage: Optional[str] = None

    customer_id: Optional[str] = None

    description: Optional[str] = None

    environment_id: Optional[str] = None

    id: Optional[str] = None

    metadata: Optional[Dict[str, str]] = None

    name: Optional[str] = None

    real_time_balance: Optional[str] = None

    real_time_credit_balance: Optional[str] = None

    status: Optional[Status] = None

    tenant_id: Optional[str] = None

    topup_conversion_rate: Optional[str] = None
    r"""topup_conversion_rate is the conversion rate for the topup to the currency
    ex if topup_conversion_rate is 1, then 1 USD = 1 credit
    ex if topup_conversion_rate is 2, then 1 USD = 0.5 credits
    ex if topup_conversion_rate is 0.5, then 1 USD = 2 credits
    """

    unpaid_invoices_amount: Optional[str] = None

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    wallet_status: Optional[WalletStatus] = None

    wallet_type: Optional[WalletType] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "alert_settings",
                "alert_state",
                "auto_topup",
                "balance",
                "balance_updated_at",
                "config",
                "conversion_rate",
                "created_at",
                "created_by",
                "credit_balance",
                "credits_available_breakdown",
                "currency",
                "current_period_usage",
                "customer_id",
                "description",
                "environment_id",
                "id",
                "metadata",
                "name",
                "real_time_balance",
                "real_time_credit_balance",
                "status",
                "tenant_id",
                "topup_conversion_rate",
                "unpaid_invoices_amount",
                "updated_at",
                "updated_by",
                "wallet_status",
                "wallet_type",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.models.components import (
    dto_triggerforcerunrequest as components_dto_triggerforcerunrequest,
)
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from flexprice_py.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TriggerScheduledTaskRunRequestTypedDict(TypedDict):
    id: str
    r"""Scheduled Task ID"""
    body: NotRequired[
        components_dto_triggerforcerunrequest.DtoTriggerForceRunRequestTypedDict
    ]
    r"""Optional start and end time for custom range"""


class TriggerScheduledTaskRunRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Scheduled Task ID"""

    body: Annotated[
        Optional[components_dto_triggerforcerunrequest.DtoTriggerForceRunRequest],
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ] = None
    r"""Optional start and end time for custom range"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["body"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .s3jobconfig import S3JobConfig, S3JobConfigTypedDict
from .scheduledtaskentitytype import ScheduledTaskEntityType
from .scheduledtaskinterval import ScheduledTaskInterval
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoScheduledTaskResponseTypedDict(TypedDict):
    connection_id: NotRequired[str]
    created_at: NotRequired[str]
    enabled: NotRequired[bool]
    entity_type: NotRequired[ScheduledTaskEntityType]
    environment_id: NotRequired[str]
    id: NotRequired[str]
    interval: NotRequired[ScheduledTaskInterval]
    job_config: NotRequired[S3JobConfigTypedDict]
    status: NotRequired[str]
    tenant_id: NotRequired[str]
    updated_at: NotRequired[str]


class DtoScheduledTaskResponse(BaseModel):
    connection_id: Optional[str] = None

    created_at: Optional[str] = None

    enabled: Optional[bool] = None

    entity_type: Optional[ScheduledTaskEntityType] = None

    environment_id: Optional[str] = None

    id: Optional[str] = None

    interval: Optional[ScheduledTaskInterval] = None

    job_config: Optional[S3JobConfig] = None

    status: Optional[str] = None

    tenant_id: Optional[str] = None

    updated_at: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "connection_id",
                "created_at",
                "enabled",
                "entity_type",
                "environment_id",
                "id",
                "interval",
                "job_config",
                "status",
                "tenant_id",
                "updated_at",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pausemode import PauseMode
from .pausestatus import PauseStatus
from .resumemode import ResumeMode
from .status import Status
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoSubscriptionPauseResponseTypedDict(TypedDict):
    r"""Response object containing subscription pause information"""

    created_at: NotRequired[str]
    created_by: NotRequired[str]
    environment_id: NotRequired[str]
    r"""EnvironmentID is the environment identifier for the pause"""
    id: NotRequired[str]
    r"""ID is the unique identifier for the subscription pause"""
    metadata: NotRequired[Dict[str, str]]
    original_period_end: NotRequired[str]
    r"""OriginalPeriodEnd is the end of the billing period when the pause was created"""
    original_period_start: NotRequired[str]
    r"""OriginalPeriodStart is the start of the billing period when the pause was created"""
    pause_end: NotRequired[str]
    r"""PauseEnd is when the pause will end (null for indefinite)"""
    pause_mode: NotRequired[PauseMode]
    pause_start: NotRequired[str]
    r"""PauseStart is when the pause actually started"""
    pause_status: NotRequired[PauseStatus]
    reason: NotRequired[str]
    r"""Reason is the reason for pausing"""
    resume_mode: NotRequired[ResumeMode]
    resumed_at: NotRequired[str]
    r"""ResumedAt is when the pause was actually ended (if manually resumed)"""
    status: NotRequired[Status]
    subscription_id: NotRequired[str]
    r"""SubscriptionID is the identifier for the subscription"""
    tenant_id: NotRequired[str]
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]


class DtoSubscriptionPauseResponse(BaseModel):
    r"""Response object containing subscription pause information"""

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    environment_id: Optional[str] = None
    r"""EnvironmentID is the environment identifier for the pause"""

    id: Optional[str] = None
    r"""ID is the unique identifier for the subscription pause"""

    metadata: Optional[Dict[str, str]] = None

    original_period_end: Optional[str] = None
    r"""OriginalPeriodEnd is the end of the billing period when the pause was created"""

    original_period_start: Optional[str] = None
    r"""OriginalPeriodStart is the start of the billing period when the pause was created"""

    pause_end: Optional[str] = None
    r"""PauseEnd is when the pause will end (null for indefinite)"""

    pause_mode: Optional[PauseMode] = None

    pause_start: Optional[str] = None
    r"""PauseStart is when the pause actually started"""

    pause_status: Optional[PauseStatus] = None

    reason: Optional[str] = None
    r"""Reason is the reason for pausing"""

    resume_mode: Optional[ResumeMode] = None

    resumed_at: Optional[str] = None
    r"""ResumedAt is when the pause was actually ended (if manually resumed)"""

    status: Optional[Status] = None

    subscription_id: Optional[str] = None
    r"""SubscriptionID is the identifier for the subscription"""

    tenant_id: Optional[str] = None

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "created_at",
                "created_by",
                "environment_id",
                "id",
                "metadata",
                "original_period_end",
                "original_period_start",
                "pause_end",
                "pause_mode",
                "pause_start",
                "pause_status",
                "reason",
                "resume_mode",
                "resumed_at",
                "status",
                "subscription_id",
                "tenant_id",
                "updated_at",
                "updated_by",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriptionschedulechangetype import SubscriptionScheduleChangeType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoCancelScheduleRequestTypedDict(TypedDict):
    r"""Request to cancel a subscription schedule (supports two modes)"""

    schedule_id: NotRequired[str]
    r"""schedule_id is the ID of the schedule to cancel (optional if subscription_id and schedule_type are provided)"""
    schedule_type: NotRequired[SubscriptionScheduleChangeType]
    subscription_id: NotRequired[str]
    r"""subscription_id is the ID of the subscription (required if schedule_id is not provided)"""


class DtoCancelScheduleRequest(BaseModel):
    r"""Request to cancel a subscription schedule (supports two modes)"""

    schedule_id: Optional[str] = None
    r"""schedule_id is the ID of the schedule to cancel (optional if subscription_id and schedule_type are provided)"""

    schedule_type: Optional[SubscriptionScheduleChangeType] = None

    subscription_id: Optional[str] = None
    r"""subscription_id is the ID of the subscription (required if schedule_id is not provided)"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["schedule_id", "schedule_type", "subscription_id"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from typing import Any, TYPE_CHECKING

from flexprice_py.utils.dynamic_imports import lazy_getattr, lazy_dir

if TYPE_CHECKING:
    from . import components, operations

# package


_sub_packages = ["components", "operations"]


def __getattr__(attr_name: str) -> Any:
    return lazy_getattr(
        attr_name, package=__package__, dynamic_imports={}, sub_packages=_sub_packages
    )


def __dir__():
    return lazy_dir(dynamic_imports={}, sub_packages=_sub_packages)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoFeatureUsageInfoTypedDict(TypedDict):
    customer_id: NotRequired[str]
    feature_id: NotRequired[str]
    meter_id: NotRequired[str]
    price_id: NotRequired[str]
    processed_at: NotRequired[str]
    qty_total: NotRequired[str]
    sub_line_item_id: NotRequired[str]
    subscription_id: NotRequired[str]


class DtoFeatureUsageInfo(BaseModel):
    customer_id: Optional[str] = None

    feature_id: Optional[str] = None

    meter_id: Optional[str] = None

    price_id: Optional[str] = None

    processed_at: Optional[str] = None

    qty_total: Optional[str] = None

    sub_line_item_id: Optional[str] = None

    subscription_id: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "customer_id",
                "feature_id",
                "meter_id",
                "price_id",
                "processed_at",
                "qty_total",
                "sub_line_item_id",
                "subscription_id",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import UnrecognizedStr
from typing import Literal, Union


PaymentTerms = Union[
    Literal[
        "15 NET",
        "30 NET",
        "45 NET",
        "60 NET",
        "75 NET",
        "90 NET",
    ],
    UnrecognizedStr,
]
